export P=gdal
export V=3.0.3
export B=2
export MAINTAINER=JuergenFischer

# Other Sources:
# https://raw.githubusercontent.com/Esri/file-geodatabase-api/master/FileGDB_API_1.5/FileGDB_API_1_5_VS2015.zip
# http://downloads.hexagongeospatial.com/software/2016/ECWJP2SDK/erdas-ecw-sdk-5.3.0-win.zip
# http://bin.lizardtech.com/download/developer/MrSID_DSDK-9.5.4.4703-win32-vc14.zip
# http://bin.lizardtech.com/download/developer/MrSID_DSDK-9.5.4.4703-win64-vc14.zip
# http://go.microsoft.com/fwlink/?LinkID=239647&clcid=0x409 => sqlncli-x86.msi
# http://go.microsoft.com/fwlink/?LinkID=239648&clcid=0x409 => sqlncli-x64.msi
# msiexec /i sqlncli.msi ADDLOCAL=ALL APPGUID={0CC618CE-F36A-415E-84B4-FB1BFF6967E1} (with SDK)

savelog() {
	local f=$1
	rm -f $f.9
	for i in $(seq 8 -1 0); do [ -f "$f.$i" ] && mv $f.$i $f.$(( i + 1 )); done
	[ -f $f ] && mv $f $f.0
}

mkdir -p log
savelog log/package.log
exec > >(tee log/package.log) 2>&1
date

set -e

PYTHON2=Python27
PYTHON3=Python37

[ -f gdal-$V.tar.gz ] || wget http://download.osgeo.org/gdal/${V%rc*}/gdal-$V.tar.gz
[ -f ../makefile.vc ] || tar -C .. -xzf gdal-$V.tar.gz --xform s,gdal-${V%rc*},.,
[ -n "$GDALDEPS" ] || export GDALDEPS=$(cygpath -am ../../gdaldeps)
[ -d $GDALDEPS ]

if ! [ -d "$OSGEO4W_REP" ]; then
        echo no repository
        exit 1
fi

# override msys
cat() { /bin/cat "$@"; }
cp() { /bin/cp "$@"; }
rm() { /bin/rm "$@"; }
mkdir() { /bin/mkdir "$@"; }
tar() { /bin/tar "$@"; }
ls() { /bin/ls "$@"; }
sed() { /bin/sed "$@"; }
diff() { /bin/diff "$@"; }
find() { /bin/find "$@"; }
xargs() { /bin/xargs "$@"; }
declare -x cat cp rm mkdir tar ls sed diff

#
# run batch file and source environment changes
#
fetchenv() {
        local IFS
        IFS=
        batch=$1
        shift

        if ! [ -f "$batch" ]; then
                echo "$batch not found"
                return 1
        fi

        srcenv=$(mktemp /tmp/srcenv.XXXXXXXXXX)
        dstenv=$(mktemp /tmp/dstenv.XXXXXXXXXX)
        diffenv=$(mktemp /tmp/diffenv.XXXXXXXXXX)
        envdiff=$(mktemp /tmp/envdiff.XXXXXXXXXX)
        cmd /c set >$srcenv
        cmd /c call $(cygpath -w $batch) "$@" \>nul 2\>nul \& set >$dstenv
        cat <<EOF >$envdiff
/^+++/d;
/^[^+]/d;
s/^+//;
# s/\\\\/\\\\\\\\/g;
/^PATH=/ {
        s/\\\\/\\//g;
        s#\\([=;]\\)\\([a-zA-Z]\\):#\\1/cygdrive/\\2#g;
        s/;/:/g;
}
s/;/\\\\;/g;
s/^/export /;
s/=/='/;
s/\\\\r//g;
s/\r$/'/;
EOF
        diff -u $srcenv $dstenv | sed -f $envdiff >$diffenv
        . $diffenv
        PATH=$PATH:/bin:/usr/bin:/cygdrive/c/windows32/system32:/cygdrive/c/windows
        rm -f $srcenv $dstenv $diffenv $envdiff
}

for ARCH in ${ARCHS:-x86 x86_64}; do
	export ARCH

	(
		case "$ARCH" in
		x86)
			export OSGEO4W_ROOT="C:\\OSGeo4W"
			export BITS=32
			export VCARCH=x86
			export ECWARCH=win32
			export FGDBARCH=
			export PNG=libpng-vc14
			;;

		x86_64)
			export OSGEO4W_ROOT="C:\\OSGeo4W64"
			export BITS=64
			export VCARCH=x64
			export ECWARCH=x64
			export FGDBARCH=64
			export PNG=libpng
			;;

		*)
			echo "Unsupported arch $ARCH" >&2
			exit 1
			;;
		esac

		fetchenv $OSGEO4W_ROOT/bin/o4w_env.bat

		R=$OSGEO4W_REP/$ARCH/release/$P
		for i in python csharp oracle filegdb ecw mrsid sosi mss hdf5; do
			[ -d $R/$P-$i ] || mkdir -p $R/$P-$i
		done
		[ -d $R/python3-$P ] || mkdir -p $R/python3-$P

		if [ -f $R/$P-$V-$B-src.tar.bz2 ]; then
			echo "$R/$P-$V-$B-src.tar.bz2 already exists - skipping"
			continue
		fi

		export EXT_NMAKE_OPT=$PWD/nmake$BITS.opt
		export FGDB_SDK=$GDALDEPS/filegdb
		export ECW_SDK=$GDALDEPS/ecw-5
		export MRSID_SDK=$GDALDEPS/MrSID_DSDK-9.5.4.4703-win$BITS-vc14
		export SQLNCLI_DIR=$GDALDEPS/mssql11-sdk
		export POPPLER=$GDALDEPS/poppler

		export DESTDIR=$PWD/install$BITS
		export PY2DESTDIR=$PWD/pyinstall2-$BITS
		export PY3DESTDIR=$PWD/pyinstall3-$BITS

		rm -rf $DESTDIR $PY2DESTDIR $PY3DESTDIR

		mkdir -p $DESTDIR/etc/ini
		mkdir -p $DESTDIR/share/gdal

		fetchenv /c/Program\ Files*/Microsoft\ Visual\ Studio\ 14.0/VC/vcvarsall.bat $VCARCH
		PATH=/cygdrive/c/Program\ Files*/Microsoft\ Visual\ Studio\ 14.0/Common7/IDE:$PATH

		cat <<EOF >$DESTDIR/etc/ini/$P.bat
SET GDAL_DATA=%OSGEO4W_ROOT%\\share\\gdal
SET GDAL_DRIVER_PATH=%OSGEO4W_ROOT%\\bin\\gdalplugins
EOF

		mkdir -p $PY2DESTDIR/etc/postinstall
		cat <<EOF >$PY2DESTDIR/etc/postinstall/$P-python.bat
"%OSGEO4W_ROOT%\\bin\\make-bat-for-py.bat"
EOF

		mkdir -p $PY3DESTDIR/etc/preremove
		cat <<EOF >$PY3DESTDIR/etc/preremove/python3-gdal.bat
call %OSGEO4W_ROOT%\\bin\\py3_env.bat
python -B %PYTHONHOME%\\Scripts\\preremove-cached.py python3-gdal
EOF

		cat <<EOF >$R/setup.hint
sdesc: "The GDAL/OGR library and commandline tools"
ldesc: "The GDAL/OGR library and commandline tools"
maintainer: $MAINTAINER
category: Libs Commandline_Utilities
requires: msvcrt2015 $PNG libjpeg proj curl geos libmysql sqlite3 netcdf libpq expat xerces-c hdf4 ogdi iconv-vc14 openjpeg spatialite freexl libkml xz zstd
EOF

		cat <<EOF >$R/$P-python/setup.hint
sdesc: "The GDAL/OGR Python Bindings and Scripts"
ldesc: "The GDAL/OGR Python Bindings and Scripts"
category: Libs
requires: $P python-core python-numpy
maintainer: $MAINTAINER
external-source: $P
EOF

		cat <<EOF >$R/python3-$P/setup.hint
sdesc: "The GDAL/OGR Python3 Bindings and Scripts"
ldesc: "The GDAL/OGR Python3 Bindings and Scripts"
category: Libs
requires: $P python3-core python3-numpy
maintainer: $MAINTAINER
external-source: $P
EOF

		cat <<EOF >$R/$P-oracle/setup.hint
sdesc: "OGR OCI and GDAL GeoRaster Plugins for Oracle"
ldesc: "OGR OCI and GDAL GeoRaster Plugins for Oracle"
category: Libs
requires: $P oci libjpeg
maintainer: $MAINTAINER
external-source: $P
EOF

		cat <<EOF >$R/$P-filegdb/setup.hint
sdesc: "OGR FileGDB Driver"
ldesc: "OGR FileGDB Driver"
category: Libs
maintainer: $MAINTAINER
requires: $P
external-source: $P
EOF

		cat <<EOF >$R/$P-ecw/setup.hint
sdesc: "ECW Raster Plugin for GDAL"
ldesc: "ECW Raster Plugin for GDAL"
category: Libs
requires: $P
maintainer: $MAINTAINER
external-source: $P
EOF

		cat <<EOF >$R/$P-mrsid/setup.hint
sdesc: "MrSID Raster Plugin for GDAL"
ldesc: "MrSID Raster Plugin for GDAL"
category: Libs
maintainer: $MAINTAINER
requires: $P
external-source: $P
EOF

		cat <<EOF >$R/$P-sosi/setup.hint
sdesc: "OGR SOSI Driver"
ldesc: "The OGR SOSI Driver enables OGR to read data in Norwegian SOSI standard (.sos)"
category: Libs
requires: $P
maintainer: $MAINTAINER
external-source: $P
EOF

		cat <<EOF >$R/$P-mss/setup.hint
sdesc: "OGR plugin with SQL Native Client support for MSSQL Bulk Copy"
ldesc: "OGR plugin with SQL Native Client support for MSSQL Bulk Copy"
category: Libs
requires: $P sqlncli
maintainer: $MAINTAINER
external-source: $P
EOF

		cat <<EOF >$R/$P-hdf5/setup.hint
sdesc: "HDF5 Plugin for GDAL"
ldesc: "HDF5 Plugin for GDAL"
category: Libs
maintainer: $MAINTAINER
requires: $P hdf5
external-source: $P
EOF

		cat <<EOF >$R/$P-csharp/setup.hint
sdesc: "The GDAL/OGR CSharp Bindings and Scripts"
ldesc: "The GDAL/OGR CSharp Bindings and Scripts"
category: Libs
requires: $P
maintainer: $MAINTAINER
external-source: $P
EOF

		cp ../LICENSE.TXT $R/$P-$V-$B.txt
		cp ../LICENSE.TXT $R/$P-oracle/$P-oracle-$V-$B.txt
		cp ../LICENSE.TXT $R/$P-mss/$P-mss-$V-$B.txt
		cp ../LICENSE.TXT $R/$P-sosi/$P-sosi-$V-$B.txt
		cp ../LICENSE.TXT $R/$P-python/$P-python-$V-$B.txt
		cp ../LICENSE.TXT $R/python3-$P/python3-$P-$V-$B.txt
		cp ../LICENSE.TXT $R/$P-csharp/$P-csharp-$V-$B.txt
		cp $FGDB_SDK/license/userestrictions.txt $R/$P-filegdb/$P-filegdb-$V-$B.txt
		cp $ECW_SDK/EULA.rtf $R/$P-ecw/$P-ecw-$V-$B.rtf
		cp $MRSID_SDK/LICENSE.pdf $R/$P-mrsid/$P-mrsid-$V-$B.pdf

		cd ..

		for i in clean default install devinstall; do
			[ -f osgeo4w/no$i ] && continue

			nmake /f makefile.vc \
				CC=$(cygpath -w $OSGEO4W_ROOT/bin/clcache.bat) \
				CPP=$(cygpath -w $OSGEO4W_ROOT/bin/clcache.bat) \
				EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) \
				GDAL_HOME=$(cygpath -w $DESTDIR) \
				FGDB_SDK=$(cygpath -w $FGDB_SDK) \
				ECWDIR=$(cygpath -w $ECW_SDK) \
				MRSID_SDK=$(cygpath -w $MRSID_SDK) \
				$i
		done

		[ -f osgeo4w/nopackage ] && exit

		cd swig

		(
			export DISTUTILS_USE_SDK=1 MSSdk=1

			nmake /f makefile.vc SWIG=$(cygpath -w $OSGEO4W_ROOT/apps/swigwin/swig.exe) EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) clean python

			cd python
			mkdir -p $PY2DESTDIR/apps/$P/python
			python setup.py install --prefix=$(cygpath -w $PY2DESTDIR/apps/$PYTHON2) --single-version-externally-managed --record ..\\..\\osgeo4w\\log\\record-2-$ARCH.log
		)

		(
			fetchenv $OSGEO4W_ROOT/bin/py3_env.bat

			unset INCLUDE LIB

			nmake /f makefile.vc PYDIR=$(cygpath -w $OSGEO4W_ROOT/apps/$PYTHON3) SWIG=$(cygpath -w $OSGEO4W_ROOT/apps/swigwin/swig.exe) EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) clean python

			cd python

			# Fix CRLF => CRLFCR conversion.
			find build -name "*.py" -print | xargs /bin/flip -u

			mkdir -p $PY3DESTDIR/apps/$P/$PYTHON3
			python setup.py install --prefix=$(cygpath -w $PY3DESTDIR/apps/$PYTHON3) --single-version-externally-managed --record ..\\..\\osgeo4w\\log\\record-3-$ARCH.log
		)

		(
			mkdir -p $DESTDIR/apps/$P/csharp
			nmake /f makefile.vc GDAL_HOME=$(cygpath -w $DESTDIR/apps/$P/csharp) SWIG=$(cygpath -w $OSGEO4W_ROOT/apps/swigwin1/swig.exe) EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) clean csharp csharp_install
		)

		cd python
		mkdir -p $PY2DESTDIR/bin
		cp scripts/*.py $PY2DESTDIR/bin
		mkdir -p $PY3DESTDIR/apps/$PYTHON3/Scripts
		cp scripts/*.py $PY3DESTDIR/apps/$PYTHON3/Scripts

		cd ../../osgeo4w

		cp $FGDB_SDK/bin$FGDBARCH/FileGDBAPI.dll $DESTDIR/bin
		cp $ECW_SDK/bin/vc140/$ECWARCH/NCSEcw.dll $DESTDIR/bin
		cp $MRSID_SDK/Raster_DSDK/lib/lti_dsdk_cdll_9.5.dll $DESTDIR/bin
		cp $MRSID_SDK/Raster_DSDK/lib/tbb.dll $DESTDIR/bin
		cp $MRSID_SDK/Raster_DSDK/lib/lti_dsdk_9.5.dll $DESTDIR/bin
		cp $MRSID_SDK/Lidar_DSDK/lib/lti_lidar_dsdk_1.1.dll $DESTDIR/bin

		tar -C $PY2DESTDIR --exclude="*.pyc" -cjvf $R/$P-python/$P-python-$V-$B.tar.bz2 \
			apps/$PYTHON2 bin etc
		tar -C $PY3DESTDIR --exclude="*.pyc" --exclude __pycache__ -cjvf $R/python3-$P/python3-$P-$V-$B.tar.bz2 \
			apps/$PYTHON3
		tar -C install$BITS --remove-files -cjvf $R/$P-filegdb/$P-filegdb-$V-$B.tar.bz2 \
			bin/gdalplugins/ogr_FileGDB.dll \
			bin/FileGDBAPI.dll
		tar -C install$BITS --remove-files -cjvf $R/$P-sosi/$P-sosi-$V-$B.tar.bz2 \
			bin/gdalplugins/ogr_SOSI.dll
		tar -C install$BITS --remove-files -cjvf $R/$P-oracle/$P-oracle-$V-$B.tar.bz2 \
			bin/gdalplugins/gdal_GEOR.dll \
			bin/gdalplugins/ogr_OCI.dll
		tar -C install$BITS --remove-files -cjvf $R/$P-mss/$P-mss-$V-$B.tar.bz2 \
			bin/gdalplugins/ogr_MSSQLSpatial.dll
		tar -C install$BITS --remove-files -cjvf $R/$P-ecw/$P-ecw-$V-$B.tar.bz2 \
			bin/gdalplugins/gdal_ECW_JP2ECW.dll \
			bin/NCSEcw.dll
		tar -C install$BITS --remove-files -cjvf $R/$P-mrsid/$P-mrsid-$V-$B.tar.bz2 \
			bin/gdalplugins/gdal_MG4Lidar.dll \
			bin/gdalplugins/gdal_MrSID.dll \
			bin/lti_dsdk_cdll_9.5.dll \
			bin/lti_dsdk_9.5.dll \
			bin/lti_lidar_dsdk_1.1.dll \
			bin/tbb.dll
		tar -C install$BITS --remove-files -cjvf $R/$P-hdf5/$P-hdf5-$V-$B.tar.bz2 \
			bin/gdalplugins/gdal_HDF5.dll
		tar -C install$BITS --remove-files -cjvf $R/$P-csharp/$P-csharp-$V-$B.tar.bz2 \
			apps/$P/csharp
		tar -C install$BITS --remove-files -cjvf $R/$P-$V-$B.tar.bz2 \
			bin \
			etc \
			include \
			lib \
			share
		tar -C .. -cjvf $R/$P-$V-$B-src.tar.bz2 \
			osgeo4w/package.sh \
			osgeo4w/nmake32.opt \
			osgeo4w/nmake64.opt
	)

	rm -f noclean
done
